/*
 *   This programm is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * @ Author: Alexander Hausner (Alexander.Hausner@web.de)
 *
 */

#include "fileiface.h"

#include <qfile.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <qpe/qpeapplication.h>

FileIFace::FileIFace() {
}

FileIFace::~FileIFace() {
}

/* read .vexedrc-file and get the actual level or load default level */
Level FileIFace::getActLevel() {
    QString level = "Classic Levels";
    int levelNum = 0;
    QFile f(LOGFILE);
    // file exists. the first line is the line with the actual level
    if (f.exists()) {
	f.open(IO_ReadOnly);
	QTextStream stream(&f);
	QString s = stream.readLine();
	QStringList slist = QStringList::split(":", s);
	level = *(slist.at(0));
	levelNum = (*(slist.at(1))).toInt();
	f.close();
    }
    // write entry in .vexedrc file
    levelNum = saveStatus(level, levelNum);

    return (this -> loadLevel(level, levelNum));
}

/* read .vexedrc-file and get the actual level or load default level */
Level FileIFace::getNewLevel(int num) {
    QFile f(LOGFILE);
    // file exists. the first line is the line with the actual level
    f.open(IO_ReadOnly);
    QTextStream stream(&f);
    QString s = stream.readLine();
    QStringList slist = QStringList::split(":", s);
    QString level = *(slist.at(0));
    int levelNum = (*(slist.at(1))).toInt();
    f.close();
    // write entry in .vexedrc file
    if (num == levelNum) {
	levelNum = saveStatus(level, levelNum+1);
	return (this -> loadLevel(level, levelNum));
    }
    return (this -> loadLevel(level, num+1));
}

/* load level from a '*.lvl'-file */
Level FileIFace::loadLevel(QString& levelFile, int levelNum) {
    Level ret(levelFile, levelNum);
    QFile f(QPEApplication::qpeDir()+QString(LVLPATH)+levelFile+".lvl");
    f.open(IO_ReadOnly);
    QTextStream stream(&f);

    // go to the n-th Level in the file
    int cnt=0;
    QString s,n;
    while (cnt <= levelNum) {
	s = stream.readLine();
	if (s.startsWith("[Level]"))
	cnt++;
    }
    // the next line in the file contains the 'board=...' entry
    s = stream.readLine();
    stream.readLine();
    // two lines below is the name=... entry
    n = stream.readLine();
    f.close();
    // remove 'board='
    s = s.right(s.length()-6);
    // remove 'name='
    n = n.right(n.length()-6);
    // parse the remaining line
    parseLine(s, &ret);
    ret.setName(n);
    return ret;
}

/* parse the line that describes a level */
void FileIFace::parseLine(QString& line, Level *level) {
    int row, col;
    QStringList list;
    // get tokens
    list = QStringList::split('/', line);
    // each token represents one row in the game
    row = 0;
    for (QStringList::Iterator iter = list.begin(); iter != list.end(); iter++) {
	col = 0;
	for (unsigned int i=0; i<(*iter).length(); i++) {
	    bool ok;
	    int val;
	    val = (*iter).mid(i,1).toInt(&ok);
	    if (ok==true) {
		if ((i+1) < (*iter).length()) {
		    val = (*iter).mid(i,2).toInt(&ok);
		    if (ok == true) {
			level->setField(row, col, WALL, val);
			i++;
			continue;
		    }
		}
		val = (*iter).mid(i,1).toInt(&ok);
		level->setField(row, col, WALL, val);
		col += val;
	    } else {
		QString tmp = (*iter).mid(i,1);
		if (tmp == "~") {
		    level->setField(row,col, EMPTY, 1);
		} else if (tmp == "a") {
		    level->setField(row,col, STONE_A, 1);
		} else if (tmp == "b") {
		    level->setField(row,col, STONE_B, 1);
		} else if (tmp == "c") {
		    level->setField(row,col, STONE_C, 1);
		} else if (tmp == "d") {
		    level->setField(row,col, STONE_D, 1);
		} else if (tmp == "e") {
		    level->setField(row,col, STONE_E, 1);
		} else if (tmp == "f") {
		    level->setField(row,col, STONE_F, 1);
		} else if (tmp == "g") {
		    level->setField(row,col, STONE_G, 1);
		} else if (tmp == "h") {
		    level->setField(row,col, STONE_H, 1);
		}
		col++;
	    }
	}
	row++;
    }
}

/* save the status in the .vexedrc-file */
int FileIFace::saveStatus(QString& fileName, int num) {
    bool ret=false;
    QFile f(LOGFILE);
    QTextStream stream;
    QString line;

    // check whether the level still exists in the *lvl-file
    QFile lvl(QPEApplication::qpeDir()+QString(LVLPATH)+fileName+".lvl");
    stream.setDevice(&lvl);
    lvl.open(IO_ReadOnly);
    int cnt=0;
    while ((line = stream.readLine()) != stream.eof()) {
	if (line.startsWith("[Level]")) {
	    // make sure that cnt is not greater than the highest possible
	    // level in the current level-pack
	    if (cnt == num) {
		ret = true;
		break;
	    }
	    cnt++;
	}
    }
    lvl.close();
    if (!ret) {
	num--;
    }

    // update .vexedrc-file
    QStringList rest,tmp;
    bool entryExists=false;
    stream.setDevice(&f);
    if (f.exists()) {
	f.open(IO_ReadOnly);
	while ((line = stream.readLine()) != NULL) {
	    tmp = QStringList::split(":", line);
	    if (*(tmp.at(0)) == fileName) {
		entryExists = true;
		break;
	    }
	    rest.append(line);
	}
	while (!stream.eof()) {
	    rest.append(stream.readLine());
	}
	f.close();

	f.open(IO_WriteOnly);
	if (entryExists) {
	    stream << *(tmp.at(0)) << ":" << num << endl;
	    for (QStringList::Iterator iter=rest.begin(); iter!=rest.end(); iter++) {
		stream << *iter << endl;
	    }
	} else {
	    stream << fileName << ":" << 0 << endl;
	    for (QStringList::Iterator iter=rest.begin(); iter!=rest.end(); iter++) {
		stream << *iter << endl;
	    }
	}
	f.close();
    } else {
	f.open(IO_WriteOnly);
	stream << fileName << ":0" << endl;
	f.close();
    }

    return num;
}

/* read .vexedrc-file and get the next level */
Level FileIFace::getLevel(const QString& name, int num) {
    QString dummy = name;
    QString level;
    int levelNum=0;
    bool falseNum=false;
    QFile f(LOGFILE);
    f.open(IO_ReadOnly);
    QTextStream stream(&f);
    QString s;
    while ((s = stream.readLine()) != NULL) {
	QStringList slist = QStringList::split(":", s);
	level = *(slist.at(0));
	levelNum = (*(slist.at(1))).toInt();
	// found entry
	if (level == dummy) {
	    if (num <= levelNum) {
		saveStatus(level, levelNum);
		return (this -> loadLevel(dummy, num));
	    }
	    falseNum=true;
	    break;
	}
    }
    // write entry in .vexedrc file
    if (falseNum) {
	levelNum = saveStatus(dummy, levelNum);
	return (this -> loadLevel(dummy, levelNum));
    }
    // new entry
    saveStatus(dummy, 0);
    return (this -> loadLevel(dummy, 0));
}

