/*
 *   This programm is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * @ Author: Alexander Hausner (Alexander.Hausner@web.de)
 *
 */

#include "mainWidget.h"

#include <qlayout.h>
#include <qhbox.h>
#include <qpe/qpeapplication.h>

MainWidget::MainWidget(QWidget *parent, const char *name) : QWidget(parent, name) {
    this -> setBackgroundMode(PaletteButton);
    this -> setCaption("QPE-Vexed");

    QVBoxLayout *qvbl = new QVBoxLayout(this);
    menu = new MenuBar(this);
    connect(menu, SIGNAL(loadLevelPack(const QString&)), this, SLOT(loadLevelFromMenu(const QString&)));
    board = new PlayingBoard(this);
    connect(board, SIGNAL(levelFinished()), this, SLOT(nextLevel()));
    qvbl -> addWidget(menu);
    qvbl -> addWidget(board);

    file = new FileIFace();
    level = file->getActLevel();
    board -> setBoard(level);

    menu -> selectEntry(level.pack());

    statuslabel = new StatusLabel(this);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
    qvbl -> addWidget(statuslabel);

    QHBox *qhb = new QHBox(this);
    qhb -> setBackgroundMode(PaletteButton);
    first = new Button(QPEApplication::qpeDir()+QString(PATH)+"first_button.bmp", qhb);
    back = new Button(QPEApplication::qpeDir()+QString(PATH)+"back_button.bmp", qhb);
    restart = new Button(QPEApplication::qpeDir()+QString(PATH)+"restart_button.bmp", qhb);
    undo = new Button(QPEApplication::qpeDir()+QString(PATH)+"undo_button.bmp", qhb);
    forward = new Button(QPEApplication::qpeDir()+QString(PATH)+"forward_button.bmp", qhb);
    last = new Button(QPEApplication::qpeDir()+QString(PATH)+"last_button.bmp", qhb);
    connect(first, SIGNAL(pressed()), this, SLOT(toFirstLevel()));
    connect(last, SIGNAL(pressed()), this, SLOT(toLastLevel()));
    connect(undo, SIGNAL(pressed()), board, SLOT(undoMove()));
    connect(restart, SIGNAL(pressed()), this, SLOT(restartLevel()));
    connect(back, SIGNAL(pressed()), this, SLOT(backOneLevel()));
    connect(forward, SIGNAL(pressed()), this, SLOT(forwardOneLevel()));

    qvbl -> addWidget(qhb);

    qvbl -> insertSpacing(1, 20);
    qvbl -> insertSpacing(3, 10);
    qvbl -> insertSpacing(5, 10);
}

MainWidget::~MainWidget() {
}

void MainWidget::loadLevelFromMenu(const QString& s) {
    level = file -> getLevel(s, 0xffff);
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::nextLevel() {
    level = file -> getNewLevel(level.num());
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::toFirstLevel() {
    level = file -> getLevel(level.pack(), 0);
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::toLastLevel() {
    level = file -> getLevel(level.pack(), 0xffff);
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::backOneLevel() {
    if (level.num() == 0) {
	level = file -> getLevel(level.pack(), 0);
    } else {
	level = file -> getLevel(level.pack(), (level.num())-1);
    }
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::forwardOneLevel() {
    level = file -> getLevel(level.pack(), (level.num())+1);
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::restartLevel() {
    board -> setBoard(level);
    statuslabel -> setLabel(level.pack(), level.name(), level.num());
}

void MainWidget::keyPressEvent(QKeyEvent *e) {
    board -> keyPressed(e);
}
