/*
 *   This programm is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * @ Author: Alexander Hausner (Alexander.Hausner@web.de)
 *
 */

#include "menuBar.h"
#include "about.h"
#include "instruction.h"

#include <qdir.h>
#include <qpe/qpeapplication.h>

MenuBar::MenuBar(QWidget *parent, const char *name) : QWidget(parent, name) {
    this -> setBackgroundMode(PaletteButton);
    this -> setFixedSize(234,25);
    menubar = new QPEMenuBar(this);

    filePopup = new QPopupMenu(menubar);
    filePopup -> insertItem("Quit", qApp, SLOT(quit()));

    levelMenu = new QPopupMenu(menubar);
    connect(levelMenu, SIGNAL(activated(int)), this, SLOT(selectLevel(int)));
    levelMenu -> setCheckable(true);
    this -> buildLevelMenu();

    QPopupMenu *helpMenu = new QPopupMenu(menubar);
    helpMenu -> insertItem("Instructions", this, SLOT(instruction()));
    helpMenu -> insertItem("About", this, SLOT(about()));

    menubar -> insertItem("Game", filePopup);
    menubar -> insertItem("Level", levelMenu);
    menubar -> insertItem("Help", helpMenu);
}

MenuBar::~MenuBar() {
    levelIDs.clear();
}

/*
 * Iterates through the directory and adds all "*.lvl" (Level-Files)
 * to the popupmenu
 */
void MenuBar::buildLevelMenu() {
    QString s;
    int i;
    QDir dir;
    dir.setPath(QPEApplication::qpeDir()+"etc/qpe-vexed/");
    dir.setFilter(QDir::Files|QDir::Readable);
    dir.setSorting(QDir::Name);
    dir.setNameFilter("*.lvl");
    const QFileInfoList *list = dir.entryInfoList();;
    QFileInfoListIterator iter( *list );
    QFileInfo *info;
    while ((info = iter.current()) != 0) {
	s = info->fileName();
	s.truncate(s.length()-4);
	i = levelMenu->insertItem(s);
	levelIDs[i] = s;
	++iter;
    }
}

void MenuBar::selectEntry(const QString& s) {
    int id=-1;
    QMap<int,QString>::Iterator iter = levelIDs.begin();
    while (iter != levelIDs.end()) {
	if (levelMenu->isItemChecked(iter.key())) {
	    levelMenu->setItemChecked(iter.key(), false);
	}
	if (iter.data() == s) {
	    id = iter.key();
	}
	iter++;
    }
    levelMenu -> setItemChecked(id, true);
}

void MenuBar::selectLevel(int i) {
    QString s = levelIDs[i];
    selectEntry(s);
    emit loadLevelPack(s);
}

void MenuBar::instruction() {
        Instruction i(this, NULL);
        i.exec();
}

void MenuBar::about() {
        About a(this, NULL);
        a.exec();
}

