/*
 *   This programm is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * @ Author: Alexander Hausner (Alexander.Hausner@web.de)
 *
 */

#ifndef _PLAYINGBOARD_H_
#define _PLAYINGBOARD_H_

#include "stone.h"

#include <qwidget.h>
#include <qtimer.h>
#include <qrect.h>
#include <qlist.h>
#include <qstack.h>

/*
 * This class handles the playing board, input processing and movement
 */

enum direction { LEFT = -STONE_WIDTH, RIGHT = STONE_WIDTH, UP = -STONE_HEIGHT, DOWN = STONE_HEIGHT};
enum evaluation { NOT_MOVED, STONES_REMOVED, STONES_NOT_REMOVED};

class PlayingBoard : public QWidget {
    Q_OBJECT
    public:
	PlayingBoard(QWidget *parent = NULL, const char *name = NULL);
	~PlayingBoard();
	void setBoard(const Level& l);
	void keyPressed(QKeyEvent *e);
    protected:
	void mousePressEvent(QMouseEvent *e);
    private:
	bool selected,visible;
	QRect cursorPos;
	QList<Stone> stones;
	QStack<QList<Stone> > oldmove;
	QList<Stone> wall;
	QTimer *timer;
	void paintEvent(QPaintEvent *event);
	void paintStone(QPainter *p, Stone& stone);
	void paintCross(QPainter *p);
	evaluation evaluate(Stone& s, direction);
	bool remove(Stone *s);
    public slots:
	void undoMove();
    private slots:
	void timedOut();
    signals:
	void levelFinished();
};

#endif
