/*
 *   This programm is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Author: Alexander Hausner (Alexander.Hausner@web.de)
 */

#include "stone.h"
#include <qpe/qpeapplication.h>
#include <qcolor.h>

static const int edges[] = {0,1,2,3,4,5,10,11,12,13,14,15};

Stone::Stone(const QRect &pos, ITEM i) {
    sel = false;
    position = pos;
    switch (i) {
	case WALL:
	    filename = "stone_i.bmp";
	    pictureID = WALL;
	    break;
	case STONE_A:
	    filename = "stone_a.bmp";
	    pictureID = STONE_A;
	    break;
	case STONE_B:
	    filename = "stone_b.bmp";
	    pictureID = STONE_B;
	    break;
	case STONE_C:
	    filename = "stone_c.bmp";
	    pictureID = STONE_C;
	    break;
	case STONE_D:
	    filename = "stone_d.bmp";
	    pictureID = STONE_D;
	    break;
	case STONE_E:
	    filename = "stone_e.bmp";
	    pictureID = STONE_E;
	    break;
	case STONE_F:
	    filename = "stone_f.bmp";
	    pictureID = STONE_F;
	    break;
	case STONE_G:
	    filename = "stone_g.bmp";
	    pictureID = STONE_G;
	    break;
	case STONE_H:
	    filename = "stone_h.bmp";
	    pictureID = STONE_H;
	    break;
	case EMPTY:
	    break;
	default:
	    break;
    }
    picture.load(QPEApplication::qpeDir()+QString(PATH)+filename);
}

Stone::~Stone() {}

const QRect& Stone::pos() {
    return position;
}

void Stone::setPos(const QRect& p) {
    position = p;
}

QImage& Stone::image() {
    return picture;
}

void Stone::select(bool s) {
    if (sel = s) {
	int i;
	picture = picture.convertDepth(32);
	for (i=0; i<12; i++) {
	    picture.setPixel(edges[i],0, 0x00000088);
	    picture.setPixel(edges[i],1, 0x00000088);
	    picture.setPixel(edges[i],14,0x00000088);
	    picture.setPixel(edges[i],15,0x00000088);
	}
	for (i=2; i<10; i++) {
	    picture.setPixel(0, edges[i], 0x00000088);
	    picture.setPixel(1, edges[i], 0x00000088);
	    picture.setPixel(14,edges[i], 0x00000088);
	    picture.setPixel(15,edges[i], 0x00000088);
	}
    } else {
	reload();
    }
}

bool Stone::selected() {
    return sel;
}

int Stone::imageID() {
    return pictureID;
}

Stone* Stone::newStone() {
    Stone *s = new Stone(position, (ITEM)pictureID);
    s -> select(sel);
    return s;
}

void Stone::reload() {
    picture.load(QPEApplication::qpeDir()+QString(PATH)+filename);
}
